<?php
namespace WCProductsWizard;

/**
 * WCProductsWizard Admin Class
 *
 * @class Admin
 * @version 8.1.0
 */
class Admin
{
    // <editor-fold desc="Properties">
    /**
     * Global settings model variable
     * @var array
     */
    public $globalSettingsModel = [];

    /**
     * Wizard post settings model variable
     * @var array
     */
    public $postSettingsModel = [];

    /**
     * Wizard steps settings model variable
     * @var array
     */
    public $stepSettingsModel = [];
    // </editor-fold>

    // <editor-fold desc="Core">
    /** Class Constructor */
    public function __construct()
    {
        // include base slave classes
        if (!class_exists('\\WCProductsWizard\\Cart')) {
            include_once(__DIR__ . DIRECTORY_SEPARATOR . 'Cart.php');
        }

        $settingsModel = Settings::getModel();

        $this->globalSettingsModel = $settingsModel['global'];
        $this->postSettingsModel = $settingsModel['post'];
        $this->stepSettingsModel = $settingsModel['step'];

        // scripts and styles
        add_action('admin_enqueue_scripts', [$this, 'enqueueAdmin'], 9);

        // main actions
        add_action('init', [$this, 'registerPostType']);
        add_action('add_meta_boxes', [$this, 'addMetaBoxes']);
        add_action('save_post_wc_product_wizard', [$this, 'savePostAction']);
        add_action('admin_footer', [$this, 'footerAction'], 9);
        add_action('admin_notices', [$this, 'noticesAction'], 10, 2);
        add_filter('manage_wc_product_wizard_posts_columns', [$this, 'postColumnsFilter']);
        add_action('manage_wc_product_wizard_posts_custom_column', [$this, 'postCustomColumnFilter'], 10, 2);
        add_filter('plugin_action_links', [$this, 'actionLinksFilter'], 10, 2);
        add_filter('plugin_row_meta', [$this, 'metaLinksFilter'], 10, 2);

        // settings page
        add_filter('woocommerce_get_settings_pages', [$this, 'addSettingsTab']);

        // ajax
        add_action('wp_ajax_wcpwGetStepSettingsForm', [$this, 'getStepSettingsAjax']);
        add_action('wp_ajax_wcpwSaveStepSettings', [$this, 'saveStepSettingsAjax']);
    }

    /** Register wizard post-type action */
    public function registerPostType()
    {
        $name = L10N::r('Products Wizard');
        $args = [
            'label' => $name,
            'labels' => [
                'name' => $name,
                'singular_name' => $name,
                'menu_name' => $name
            ],
            'description' => L10N::r('This is where you can add new products wizard items.'),
            'public' => false,
            'show_ui' => true,
            'map_meta_cap' => true,
            'publicly_queryable' => false,
            'exclude_from_search' => true,
            'show_in_menu' => current_user_can('manage_woocommerce') ? 'woocommerce' : true,
            'hierarchical' => false,
            'rewrite' => false,
            'query_var' => false,
            'supports' => [
                'title',
                'editor'
            ],
            'show_in_nav_menus' => false,
            'show_in_admin_bar' => true
        ];

        register_post_type('wc_product_wizard', apply_filters('wcpw_post_type_args', $args));
    }

    /**
     * Styles and scripts enqueue in admin.
     */
    public function enqueueAdmin()
    {
        wp_enqueue_media();
        wp_enqueue_script('jquery');
        wp_enqueue_script('jquery-ui-sortable');

        if (defined('WC_VERSION')
            && ((isset($_GET['post']) && get_post_type((int) $_GET['post']) == 'wc_product_wizard')
            || (isset($_GET['post_type']) && $_GET['post_type'] == 'wc_product_wizard'))
        ) {
            wp_register_script(
                'select2',
                \WC()->plugin_url() . "/assets/js/select2/select2.full.min.js",
                ['jquery'],
                '4.0.3'
            );

            wp_register_script(
                'selectWoo',
                WC()->plugin_url() . "/assets/js/selectWoo/selectWoo.full.min.js",
                ['jquery'],
                '1.0.0'
            );

            wp_register_script(
                'wc-enhanced-select',
                WC()->plugin_url() . "/assets/js/admin/wc-enhanced-select.min.js",
                ['jquery', 'selectWoo'],
                WC_VERSION
            );

            wp_enqueue_script('select2');
            wp_enqueue_script('selectWoo');
            wp_enqueue_script('wc-enhanced-select');
            wp_enqueue_style('woocommerce_admin_styles', \WC()->plugin_url() . '/assets/css/admin.css', [], WC_VERSION);
        }

        wp_enqueue_script(
            'wcpw-shared-editor-modal',
            WC_PRODUCTS_WIZARD_PLUGIN_URL . "assets/admin/js/shared-editor-modal.min.js",
            ['jquery'],
            WC_PRODUCTS_WIZARD_VERSION,
            true
        );

        wp_enqueue_script(
            'wcpw-steps',
            WC_PRODUCTS_WIZARD_PLUGIN_URL . "assets/admin/js/steps.min.js",
            [
                'jquery',
                'jquery-ui-sortable'
            ],
            WC_PRODUCTS_WIZARD_VERSION,
            true
        );

        wp_enqueue_script(
            'wcpw-multi-select',
            WC_PRODUCTS_WIZARD_PLUGIN_URL . "assets/admin/js/multi-select.min.js",
            ['jquery'],
            WC_PRODUCTS_WIZARD_VERSION,
            true
        );

        wp_enqueue_script(
            'wcpw-hooks',
            WC_PRODUCTS_WIZARD_PLUGIN_URL . "assets/admin/js/hooks.min.js",
            'jquery',
            WC_PRODUCTS_WIZARD_VERSION,
            true
        );

        wp_enqueue_style(
            'wcpw-app',
            WC_PRODUCTS_WIZARD_PLUGIN_URL . "assets/admin/css/app.min.css",
            [],
            WC_PRODUCTS_WIZARD_VERSION
        );
    }

    /** Admin notices hook */
    public function noticesAction()
    {
        global $pagenow;

        if ($pagenow == 'post.php' && isset($_GET['post']) && get_post_type($_GET['post']) == 'wc_product_wizard') {
            ?>
            <div class="notice notice-success is-dismissible">
                <h2><?php L10N::e('Unlock more features!'); ?></h2>
                <?php L10N::e('Use super-power and all possibilities of the wizard with the full version.'); ?>
                <p>
                    <a class="button button-primary" href="https://products-wizard.troll-winner.com/plugin-shop-page/"
                        target="_blank" rel="nofollow">&nbsp;&nbsp;<?php L10N::e('Go PRO!'); ?>&nbsp;&nbsp;
                    </a>
                </p>
            </div>
            <?php
        }
    }

    /**
     * Generate html-field from passed args
     *
     * @param array $modelItem
     * @param array $args
     */
    public static function settingFieldView($modelItem, $args = [])
    {
        $defaultArgs = [
            'values' => [],
            'namePattern' => '%key%',
            'idPattern' => '%key%',
            'generateId' => true,
            'asTemplate' => false
        ];

        $args = array_replace($defaultArgs, $args);

        // create name from pattern
        $args['name'] = str_replace('%key%', $modelItem['key'], $args['namePattern']);

        // define value
        $args['value'] = isset($args['values'][$modelItem['key']])
            ? $args['values'][$modelItem['key']]
            : (isset($modelItem['default']) ? $modelItem['default'] : '');

        // define id attribute
        $args['id'] = $args['generateId'] ? str_replace('%key%', $modelItem['key'], $args['idPattern']) : false;

        $viewPath = __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR
            . 'setting-field-views' . DIRECTORY_SEPARATOR . $modelItem['type'] . '.php';

        echo isset($modelItem['before']) ? $modelItem['before'] : '';

        if (file_exists($viewPath)) {
            include($viewPath);
        }

        echo isset($modelItem['after']) ? $modelItem['after'] : '';

        if (isset($modelItem['description'])) {
            echo '<p class="description">';
            echo wp_kses_post($modelItem['description']);
            echo '</p>';
        }
    }
    // </editor-fold>

    // <editor-fold desc="Posts settings">
    /**
     * Save wizard meta values
     *
     * @param integer $postId
     */
    public function savePostAction($postId)
    {
        if ((defined('DOING_AUTOSAVE') && DOING_AUTOSAVE)
            || !current_user_can('edit_page', $postId)
            || (isset($_REQUEST['action']) && $_REQUEST['action'] == 'inline-save')
        ) {
            return;
        }

        foreach ($this->postSettingsModel as $setting) {
            $value = '';

            if (isset($_POST[$setting['key']])) {
                $value = $_POST[$setting['key']];
            }

            update_post_meta($postId, $setting['key'], $value);
        }

        // update steps meta
        $stepsIds = isset($_POST['_steps_ids']) ? $_POST['_steps_ids'] : [];
        $stepsSettings = get_post_meta($postId, '_steps_settings', true);
        $stepsSettings = $stepsSettings ? $stepsSettings : [];

        update_post_meta($postId, '_steps_ids', $stepsIds);

        foreach ($stepsSettings as $stepId => $stepSettings) {
            if (!in_array($stepId, $stepsIds)) {
                unset($stepsSettings[$stepId]);
            }
        }

        update_post_meta($postId, '_steps_settings', $stepsSettings);
    }

    /** Add wizard meta-boxes */
    public function addMetaBoxes()
    {
        add_meta_box(
            'options',
            L10N::r('Options'),
            [$this, 'wizardMetaBoxCallback'],
            'wc_product_wizard',
            'normal'
        );
    }

    /** WP footer hook */
    public function footerAction()
    {
        if ((isset($_GET['post']) && get_post_type((int) $_GET['post']) == 'wc_product_wizard')
            || (isset($_GET['post_type']) && $_GET['post_type'] == 'wc_product_wizard')
        ) {
            ?>
            <div class="wcpw-modal" data-component="wcpw-step-modal">
                <div class="wcpw-modal-dialog">
                    <a href="#close"
                        title="<?php L10N::e('Close'); ?>"
                        data-component="wcpw-step-modal-close"
                        class="wcpw-modal-close">&times;</a>
                    <div class="wcpw-modal-dialog-body" data-component="wcpw-step-modal-body"></div>
                </div>
            </div>
            <?php
        }
    }

    /**
     * Products wizard post type metabox content
     *
     * @param object $post
     */
    public function wizardMetaBoxCallback($post)
    {
        $stepsIds = Settings::getStepsIds($post->ID);
        $stepsSettings = Settings::getStepsSettings($post->ID);
        $defaultSettingsUrl = [
            'action' => 'wcpwGetStepSettingsForm',
            'post_id' => $post->ID,
            'step_id' => '%STEP_ID%'
        ];

        $postSettingsModelGroups = [];

        foreach ($this->postSettingsModel as $key => $setting) {
            $postSettingsModelGroups[$setting['group']][$key] = $setting;
        }
        ?>
        <div data-component="wcpw-settings-groups">
            <fieldset class="wcpw-settings-group">
                <a class="button button-large wcpw-settings-group-toggle"
                    href="#" role="button"
                    data-component="wcpw-settings-group-toggle"
                    data-id="basic"><?php L10N::e('Basic'); ?></a>
                <table class="form-table wcpw-settings-table wcpw-settings-group-content is-visible"
                    data-component="wcpw-settings-group-content"
                    data-id="basic">
                    <tr valign="top">
                        <th scope="row">
                            <label for="woocommerce-products-wizard-shortcode"><?php
                                L10N::e('Using');
                                ?></label>
                        </th>
                        <td>
                            <input type="text" id="woocommerce-products-wizard-shortcode" readonly
                                value="<?php echo esc_attr('[woocommerce-products-wizard id="' . $post->ID . '"]'); ?>">
                        </td>
                    </tr>
                    <tr valign="top"
                        data-component="wcpw-steps"
                        data-ajax-url="<?php echo admin_url('admin-ajax.php'); ?>">
                        <th scope="row">
                            <button class="button" data-component="wcpw-steps-add"><?php
                                L10N::e('Add step');
                                ?></button>
                        </th>
                        <td>
                            <table class="wcpw-settings-table wcpw-data-table wcpw-steps-list wp-list-table
                                widefat striped">
                                <tbody data-component="wcpw-steps-list">
                                    <?php
                                    foreach ($stepsIds as $stepId) {
                                        $settingsUrl = [
                                            'action' => 'wcpwGetStepSettingsForm',
                                            'post_id' => $post->ID,
                                            'step_id' => $stepId
                                        ];
                                        ?>
                                        <tr class="wcpw-steps-list-row wcpw-data-table-item"
                                            data-component="wcpw-steps-list-item"
                                            data-id="<?php echo esc_attr($stepId); ?>">
                                            <td>
                                                <span data-component="wcpw-steps-list-item-name"><?php
                                                    echo wp_kses_post("#{$stepId} ");

                                                    if (isset($stepsSettings[$stepId], $stepsSettings[$stepId]['title'])
                                                        && !empty($stepsSettings[$stepId]['title'])
                                                    ) {
                                                        echo wp_kses_post($stepsSettings[$stepId]['title']);
                                                    }
                                                    ?></span>
                                                <input type="hidden"
                                                    data-component="wcpw-steps-list-item-id"
                                                    name="_steps_ids[<?php echo esc_attr($stepId); ?>]"
                                                    value="<?php echo esc_attr($stepId); ?>">
                                            </td>
                                            <td class="wcpw-data-table-item-controls">
                                                <button class="button wcpw-steps-list-item-settings"
                                                    data-component="wcpw-steps-list-item-settings"
                                                    data-settings="<?php
                                                    echo esc_attr(wp_json_encode($settingsUrl));
                                                    ?>"><?php L10N::e('Settings'); ?></button>
                                            </td>
                                            <td class="wcpw-data-table-item-controls">
                                                <button class="button"
                                                    data-component="wcpw-steps-list-item-remove">&times;</button>
                                            </td>
                                        </tr>
                                        <?php
                                    }
                                    ?>
                                </tbody>
                                <tfoot class="wcpw-steps-list-footer">
                                    <tr class="wcpw-steps-list-row wcpw-data-table-item"
                                        data-component="wcpw-steps-list-item-template">
                                        <td>
                                            <span data-component="wcpw-steps-list-item-name"></span>
                                            <input type="hidden"
                                                data-component="wcpw-steps-list-item-id">
                                        </td>
                                        <td class="wcpw-data-table-item-controls">
                                            <button class="button wcpw-steps-list-item-settings"
                                                data-component="wcpw-steps-list-item-settings"
                                                data-settings="<?php
                                                echo esc_attr(wp_json_encode($defaultSettingsUrl));
                                                ?>"><?php L10N::e('Settings'); ?></button>
                                        </td>
                                        <td class="wcpw-data-table-item-controls">
                                            <button class="button"
                                                data-component="wcpw-steps-list-item-remove">&times;</button>
                                        </td>
                                    </tr>
                                </tfoot>
                            </table>
                        </td>
                    </tr>
                </table>
            </fieldset>
            <?php foreach ($postSettingsModelGroups as $group => $settingsModel) { ?>
                <fieldset class="wcpw-settings-group">
                    <a class="button button-large wcpw-settings-group-toggle"
                        href="#" role="button"
                        data-component="wcpw-settings-group-toggle"
                        data-id="<?php echo esc_attr($group); ?>"><?php echo wp_kses_post($group); ?></a>
                    <table class="form-table wcpw-settings-table wcpw-settings-group-content"
                        data-component="wcpw-settings-group-content"
                        data-id="<?php echo esc_attr($group); ?>">
                        <?php foreach ($settingsModel as $key => $setting) { ?>
                            <tr valign="top">
                                <th scope="row" class="titledesc">
                                    <label for="<?php echo esc_attr($setting['key']); ?>"><?php
                                        echo wp_kses_post($setting['label']);
                                        ?></label>
                                </th>
                                <td><?php
                                    self::settingFieldView(
                                        $setting,
                                        ['values' => [$setting['key'] => Settings::getPost($post->ID, $key)]]
                                    );
                                    ?></td>
                            </tr>
                        <?php } ?>
                    </table>
                </fieldset>
            <?php } ?>
        </div>
        <?php
        include(
            __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'global' . DIRECTORY_SEPARATOR
            . 'shared-editor-modal.php'
        );
    }

    /**
     * Wizards list columns filter
     *
     * @param array $columns
     *
     * @return array $columns
     */
    public function postColumnsFilter($columns)
    {
        $columns['shortcode'] = L10N::r('ShortCode');

        return $columns;
    }

    /**
     * Wizards list line cell
     *
     * @param array $columns
     * @param integer $postId
     */
    public function postCustomColumnFilter($columns, $postId)
    {
        if ($columns == 'shortcode') {
            echo '[woocommerce-products-wizard id="' . (int) $postId . '"]';
        }
    }
    // </editor-fold>

    // <editor-fold desc="Product categories">
    /**
     * Get the tree of categories
     *
     * @param array $args
     * @param integer $level
     *
     * @return array
     */
    public static function getCategoriesTree($args = [], $level = 0)
    {
        $defaults = [
            'taxonomy' => 'product_cat',
            'hide_empty' => false,
            'hierarchical' => true,
            'parent' => 0
        ];

        $args = array_merge($defaults, $args);
        $output = [];

        foreach (get_terms($args) as $term) {
            $term->name = str_repeat('—', $level) . $term->name;
            $output[$term->term_id] = $term;
            $output = $output + self::getCategoriesTree(['parent' => $term->term_id], $level + 1);
        }

        return $output;
    }
    // </editor-fold>

    // <editor-fold desc="Steps list">
    /** Get wizard step settings form via ajax */
    public function getStepSettingsAjax()
    {
        try {
            $this->getStepSettingsForm($_GET);
        } catch (\Exception $exception) {
            exit($exception->getMessage());
        }

        exit();
    }

    /**
     * Get wizard step settings form
     *
     * @param array $args
     *
     * @throws \Exception if empty step or post id
     */
    public function getStepSettingsForm($args)
    {
        $postId = $args['post_id'];
        $stepId = $args['step_id'];

        if (!$stepId || !$postId) {
            throw new \Exception('Empty step or post id');
        }

        $settingsModelGroups = [];
        $meta = (array) Settings::getStepsSettings($postId);
        $categories = self::getCategoriesTree();

        // add categories list for using
        foreach ($categories as $category) {
            $this->stepSettingsModel['categories']['values'][$category->term_id] = $category->name;
        }

        // group settings
        foreach ($this->stepSettingsModel as $key => $setting) {
            $settingsModelGroups[$setting['group']][$key] = $setting;
        }

        do_action('wcpw_step_settings_form', $this, $args);
        ?>
        <form class="wcpw-step-settings-form"
            data-component="wcpw-step-settings-form wcpw-settings-groups"
            data-step-id="<?php echo esc_attr($stepId); ?>"
            data-post-id="<?php echo esc_attr($postId); ?>">
            <?php foreach ($settingsModelGroups as $group => $settingsModel) { ?>
                <div class="wcpw-settings-group">
                    <a class="button button-large wcpw-settings-group-toggle"
                        href="#" role="button"
                        data-component="wcpw-settings-group-toggle"
                        data-id="<?php echo esc_attr($group); ?>"><?php echo wp_kses_post($group); ?></a>
                    <table class="form-table wcpw-settings-table wcpw-settings-group-content"
                        data-component="wcpw-settings-group-content"
                        data-id="<?php echo esc_attr($group); ?>">
                        <?php foreach ($settingsModel as $setting) { ?>
                            <tr class="form-field">
                                <th scope="row">
                                    <label for="<?php echo esc_attr($setting['key']); ?>"><?php
                                        echo wp_kses_post($setting['label']);
                                        ?></label>
                                </th>
                                <td><?php
                                    self::settingFieldView(
                                        $setting,
                                        ['values' => isset($meta[$stepId]) ? $meta[$stepId] : []]
                                    );
                                    ?></td>
                            </tr>
                        <?php } ?>
                    </table>
                </div>
            <?php } ?>
            <footer class="wcpw-step-settings-form-footer">
                <button class="button button-primary button-large" type="submit"><?php
                    L10N::e('Save');
                    ?></button>
            </footer>
        </form>
        <?php
    }

    /** Save wizard step settings-form by ajax */
    public function saveStepSettingsAjax()
    {
        $args = $_POST;
        $values = [];
        parse_str($args['values'], $values);
        $args['values'] = $values;

        try {
            $this->saveStepSettings($args);
        } catch (\Exception $exception) {
            exit($exception->getMessage());
        }

        exit();
    }

    /**
     * Save wizard step settings-form
     *
     * @param array $args
     *
     * @throws \Exception if empty step or post id
     */
    public function saveStepSettings($args)
    {
        $postId = $args['post_id'];
        $stepId = $args['step_id'];

        if (!$stepId || !$postId) {
            throw new \Exception('Empty step or post id');
        }

        $settings = (array) Settings::getStepsSettings($postId);
        $settings[$stepId] = isset($args['values']) ? $args['values'] : [];

        update_post_meta($postId, '_steps_settings', $settings);
    }
    // </editor-fold>

    // <editor-fold desc="Misc">
    /**
     * Add wizard page to the woocommerce options page
     *
     * @param array $includes
     *
     * @return array
     */
    public function addSettingsTab($includes)
    {
        if (!class_exists('\\WCProductsWizard\\Settings_Page')) {
            include_once(__DIR__ . DIRECTORY_SEPARATOR . 'Settings_Page.php');
        }

        $includes[] = new Settings_Page($this->globalSettingsModel);

        return $includes;
    }

    /**
     * Plugins list meta link filter
     *
     * @param array $links
     * @param string $plugin
     *
     * @return array
     */
    public function metaLinksFilter($links, $plugin)
    {
        if (false === strpos($plugin, 'woocommerce-products-wizard.php')) {
            return $links;
        }

        $extraLinks = [
            'docs' => '<a href="https://products-wizard.troll-winner.ru/docs/" target="_blank" rel="nofollow">'
                . L10N::r('Docs') . '</a>'
        ];

        $links = array_merge($links, $extraLinks);

        return $links;
    }

    /**
     * Plugins list action links filter
     *
     * @param array $links
     * @param string $plugin
     *
     * @return array
     */
    public function actionLinksFilter($links, $plugin)
    {
        if (false === strpos($plugin, 'woocommerce-products-wizard.php')) {
            return $links;
        }

        $extraLinks = [
            'upgrade' => '<a href="https://products-wizard.troll-winner.com/plugin-shop-page/" '
                . 'target="_blank" rel="nofollow" style="font-weight:bold;color:#b32d2e">' . L10N::r('Go PRO!') . '</a>',
            'settings' => '<a href="admin.php?page=wc-settings&tab=products_wizard">'
                . L10N::r('Settings') . '</a>'
        ];

        $links = array_merge($extraLinks, $links);

        return $links;
    }
    // </editor-fold>
}
