<?php
namespace WCProductsWizard;

/**
 * WCProductsWizard Settings Class
 *
 * @class Settings
 * @version 5.3.0
 */
class Settings
{
    /**
     * Define settings model
     *
     * @return array
     */
    public static function getModel()
    {
        static $modelCache;

        if ($modelCache) {
            return apply_filters('wcpw_settings_models', $modelCache);
        }

        $global = [
            'store_session_in_db' => [
                'name' => L10N::r('Store session in the DB'),
                'default' => 'no',
                'type' => 'checkbox',
                'desc' => L10N::r('Tick in a case of problems of the wizard\'s state storing'),
                'id' => 'woocommerce_products_wizard_store_session_in_db',
                'key' => 'woocommerce_products_wizard_store_session_in_db',
                'section' => ''
            ]
        ];

        $post = [
            // <editor-fold desc="Behavior">
            'mode' => [
                'label' => L10N::r('Work mode'),
                'key' => '_mode',
                'type' => 'select',
                'values' => [
                    'free-walk' => L10N::r('Free walk'),
                    'single-step' => L10N::r('Single step')
                ],
                'default' => 'free-walk',
                'group' => L10N::r('Behavior')
            ],
            'scrolling_top_on_update' => [
                'label' => L10N::r('Scrolling to top on the form update'),
                'key' => '_scrolling_top_on_update',
                'type' => 'checkbox',
                'default' => true,
                'group' => L10N::r('Behavior'),
                'public' => true
            ],
            'scrolling_up_gap' => [
                'label' => L10N::r('The gap on scrolling up (px)'),
                'key' => '_scrolling_up_gap',
                'type' => 'number',
                'default' => 0,
                'group' => L10N::r('Behavior'),
                'public' => true
            ],
            // </editor-fold>
            // <editor-fold desc="Layout">
            'sticky_sidebar_offset_top' => [
                'label' => L10N::r('Sticky sidebar offset top'),
                'key' => '_sticky_sidebar_offset_top',
                'type' => 'number',
                'default' => 75,
                'group' => L10N::r('Layout')
            ],
            'sticky_controls_offset_top' => [
                'label' => L10N::r('Sticky controls offset top'),
                'key' => '_sticky_controls_offset_top',
                'type' => 'number',
                'default' => 0,
                'group' => L10N::r('Layout')
            ],
            // </editor-fold>
            // <editor-fold desc="Tabs">
            'description_tab_title' => [
                'label' => L10N::r('Description tab title'),
                'key' => '_description_tab_title',
                'type' => 'text',
                'default' => L10N::r('Welcome'),
                'group' => L10N::r('Tabs')
            ],
            'results_tab_title' => [
                'label' => L10N::r('Results tab title'),
                'key' => '_results_tab_title',
                'type' => 'text',
                'default' => L10N::r('Total'),
                'group' => L10N::r('Tabs')
            ],
            'results_price_string' => [
                'label' => L10N::r('Result tab price string'),
                'key' => '_results_price_string',
                'type' => 'text',
                'default' => L10N::r('Price'),
                'group' => L10N::r('Tabs')
            ],
            'results_thumbnail_string' => [
                'label' => L10N::r('Result thumbnail string'),
                'key' => '_results_thumbnail_string',
                'type' => 'text',
                'default' => L10N::r('Thumbnail'),
                'group' => L10N::r('Tabs')
            ],
            'results_data_string' => [
                'label' => L10N::r('Result data string'),
                'key' => '_results_data_string',
                'type' => 'text',
                'default' => L10N::r('Product'),
                'group' => L10N::r('Tabs')
            ],
            'results_remove_string' => [
                'label' => L10N::r('Result remove string'),
                'key' => '_results_remove_string',
                'type' => 'text',
                'default' => L10N::r('Remove'),
                'group' => L10N::r('Tabs')
            ],
            'results_quantity_string' => [
                'label' => L10N::r('Result quantity string'),
                'key' => '_results_quantity_string',
                'type' => 'text',
                'default' => L10N::r('Quantity'),
                'group' => L10N::r('Tabs')
            ],
            // </editor-fold>
            // <editor-fold desc="Strings">
            'empty_cart_message' => [
                'label' => L10N::r('Empty cart'),
                'key' => '_empty_cart_message',
                'type' => 'text',
                'default' => L10N::r('Your cart is empty'),
                'group' => L10N::r('Strings')
            ],
            'nothing_found_message' => [
                'label' => L10N::r('Nothing found'),
                'key' => '_nothing_found_message',
                'type' => 'text',
                'default' => L10N::r('No products were found matching your selection.', 'woocommerce'),
                'group' => L10N::r('Strings')
            ],
            'subtotal_string' => [
                'label' => L10N::r('Subtotal string'),
                'key' => '_subtotal_string',
                'type' => 'text',
                'default' => L10N::r('Subtotal'),
                'group' => L10N::r('Strings')
            ],
            'total_string' => [
                'label' => L10N::r('Total string'),
                'key' => '_total_string',
                'type' => 'text',
                'default' => L10N::r('Total'),
                'group' => L10N::r('Strings')
            ],
            // </editor-fold>
            // <editor-fold desc="Controls">
            'start_button_text' => [
                'label' => L10N::r('"Start" button text'),
                'key' => '_start_button_text',
                'type' => 'text',
                'default' => L10N::r('Start'),
                'group' => L10N::r('Controls')
            ],
            'enable_add_to_cart_button' => [
                'label' => L10N::r('Enable "Add to cart" button'),
                'key' => '_enable_add_to_cart_button',
                'type' => 'checkbox',
                'default' => true,
                'description' => L10N::r('Appears at the last page'),
                'group' => L10N::r('Controls')
            ],
            'add_to_cart_button_text' => [
                'label' => L10N::r('"Add to cart" button text'),
                'key' => '_add_to_cart_button_text',
                'type' => 'text',
                'default' => L10N::r('Add to cart'),
                'group' => L10N::r('Controls')
            ],
            'enable_remove_button' => [
                'label' => L10N::r('Enable "Remove" button'),
                'key' => '_enable_remove_button',
                'type' => 'checkbox',
                'default' => false,
                'description' => L10N::r('Appears in the cart tab and widget'),
                'group' => L10N::r('Controls')
            ],
            'remove_button_text' => [
                'label' => L10N::r('"Remove" button text'),
                'key' => '_remove_button_text',
                'type' => 'text',
                'default' => L10N::r('Remove'),
                'group' => L10N::r('Controls')
            ],
            'enable_back_button' => [
                'label' => L10N::r('Enable "Back" button'),
                'key' => '_enable_back_button',
                'type' => 'checkbox',
                'default' => true,
                'group' => L10N::r('Controls')
            ],
            'back_button_text' => [
                'label' => L10N::r('"Back" button text'),
                'key' => '_back_button_text',
                'type' => 'text',
                'default' => L10N::r('Back'),
                'group' => L10N::r('Controls')
            ],
            'enable_next_button' => [
                'label' => L10N::r('Enable "Next" button'),
                'key' => '_enable_next_button',
                'type' => 'checkbox',
                'default' => true,
                'group' => L10N::r('Controls')
            ],
            'next_button_text' => [
                'label' => L10N::r('"Next" button text'),
                'key' => '_next_button_text',
                'type' => 'text',
                'default' => L10N::r('Next'),
                'group' => L10N::r('Controls')
            ],
            'enable_reset_button' => [
                'label' => L10N::r('Enable "Reset" button'),
                'key' => '_enable_reset_button',
                'type' => 'checkbox',
                'default' => true,
                'group' => L10N::r('Controls')
            ],
            'reset_button_text' => [
                'label' => L10N::r('"Reset" button text'),
                'key' => '_reset_button_text',
                'type' => 'text',
                'default' => L10N::r('Reset'),
                'group' => L10N::r('Controls')
            ],
            'enable_skip_button' => [
                'label' => L10N::r('Enable "Skip" button'),
                'key' => '_enable_skip_button',
                'type' => 'checkbox',
                'default' => true,
                'group' => L10N::r('Controls')
            ],
            'skip_button_text' => [
                'label' => L10N::r('"Skip" button text'),
                'key' => '_skip_button_text',
                'type' => 'text',
                'default' => L10N::r('Skip'),
                'group' => L10N::r('Controls')
            ],
            'enable_to_results_button' => [
                'label' => L10N::r('Enable "To results" button'),
                'key' => '_enable_to_results_button',
                'type' => 'checkbox',
                'default' => true,
                'group' => L10N::r('Controls')
            ],
            'to_results_button_text' => [
                'label' => L10N::r('"To results" button text'),
                'key' => '_to_results_button_text',
                'type' => 'text',
                'default' => L10N::r('To results'),
                'group' => L10N::r('Controls')
            ],
            'widget_toggle_button_text' => [
                'label' => L10N::r('"Toggle widget" button text'),
                'key' => '_widget_toggle_button_text',
                'type' => 'text',
                'default' => L10N::r('Toggle cart'),
                'group' => L10N::r('Controls')
            ]
            // </editor-fold>
        ];

        $step = [
            // <editor-fold desc="Captions">
            'title' => [
                'label' => L10N::r('Title'),
                'key' => 'title',
                'type' => 'text',
                'default' => '',
                'group' => L10N::r('Captions')
            ],
            'description' => [
                'label' => L10N::r('Description'),
                'key' => 'description',
                'type' => 'editor',
                'inModal' => true,
                'default' => '',
                'group' => L10N::r('Captions')
            ],
            'description_position' => [
                'label' => L10N::r('Description position'),
                'key' => 'description_position',
                'type' => 'select',
                'values' => [
                    'top' => L10N::r('Top'),
                    'bottom' => L10N::r('Bottom')
                ],
                'default' => 'top',
                'group' => L10N::r('Captions')
            ],
            'description_auto_tags' => [
                'label' => L10N::r('Handle description with auto tags'),
                'key' => 'description_auto_tags',
                'type' => 'checkbox',
                'default' => true,
                'group' => L10N::r('Captions')
            ],
            'item_description_source' => [
                'label' => L10N::r('Item description source'),
                'key' => 'item_description_source',
                'type' => 'select',
                'values' => [
                    'content' => L10N::r('Product content'),
                    'excerpt' => L10N::r('Product short description'),
                    'none' => L10N::r('None')
                ],
                'default' => 'content',
                'group' => L10N::r('Captions')
            ],
            // </editor-fold>
            // <editor-fold desc="Query">
            'categories' => [
                'label' => L10N::r('Categories for using'),
                'key' => 'categories',
                'type' => 'multi-select',
                'default' => [],
                'values' => [],
                'description' => L10N::r('Select categories to get products'),
                'group' => L10N::r('Query')
            ],
            'order' => [
                'label' => L10N::r('Order'),
                'key' => 'order',
                'type' => 'select',
                'values' => [
                    'ASC' => L10N::r('ASC'),
                    'DESC' => L10N::r('DESC')
                ],
                'default' => 'ASC',
                'group' => L10N::r('Query')
            ],
            'order_by' => [
                'label' => L10N::r('Order by'),
                'key' => 'order_by',
                'type' => 'select',
                'values' => [
                    'ID' => L10N::r('ID'),
                    'author' => L10N::r('Author'),
                    'name' => L10N::r('Name'),
                    'date' => L10N::r('Date'),
                    'modified' => L10N::r('Modified'),
                    'rand' => L10N::r('Rand'),
                    'comment_count' => L10N::r('Comment count'),
                    'menu_order' => L10N::r('Menu order'),
                    'post__in' => L10N::r('Included products'),
                    'price' => L10N::r('Price')
                ],
                'default' => 'menu_order',
                'group' => L10N::r('Query')
            ],
            'products_per_page' => [
                'label' => L10N::r('Products per page'),
                'key' => 'products_per_page',
                'type' => 'number',
                'min' => 0,
                'default' => 0,
                'description' => L10N::r('Zero is equal infinity'),
                'group' => L10N::r('Query')
            ],
            // </editor-fold>
            // <editor-fold desc="Cart">
            'several_products' => [
                'label' => L10N::r('Can select several products'),
                'key' => 'several_products',
                'type' => 'checkbox',
                'default' => false,
                'description' => L10N::r('Replace radio-inputs with checkboxes'),
                'group' => L10N::r('Cart')
            ],
            'sold_individually' => [
                'label' => L10N::r('Sold individually'),
                'key' => 'sold_individually',
                'type' => 'checkbox',
                'default' => false,
                'description' => L10N::r('Hide products quantity input'),
                'group' => L10N::r('Cart')
            ],
            // </editor-fold>
            // <editor-fold desc="View">
            'template' => [
                'label' => L10N::r('Template'),
                'key' => 'template',
                'type' => 'select',
                'values' => [
                    'grid' => 'grid',
                    'list' => 'list'
                ],
                'default' => 'list',
                'group' => L10N::r('View')
            ],
            'item_template' => [
                'label' => L10N::r('Item template'),
                'key' => 'item_template',
                'type' => 'select',
                'values' => [
                    'type-1' => 'type-1',
                    'type-2' => 'type-2'
                ],
                'default' => 'type-1',
                'group' => L10N::r('View')
            ],
            // </editor-fold>
        ];

        $modelCache = [
            'global' => apply_filters('wcpw_global_settings_model', $global),
            'post' => apply_filters('wcpw_post_settings_model', $post),
            'step' => apply_filters('wcpw_step_settings_model', $step)
        ];

        return apply_filters('wcpw_settings_models', $modelCache);
    }

    /**
     * Handle the setting value according to the type
     *
     * @param mixed $value
     * @param string $type
     *
     * @return string|float|bool|array
     */
    public static function handleSettingType($value, $type = 'string')
    {
        switch ($type) {
            case 'checkbox':
                $value = filter_var($value, FILTER_VALIDATE_BOOLEAN);
                break;

            case 'number':
                $value = (float) $value;
                break;

            case 'array':
                $value = (array) $value;
                break;
        }

        return $value;
    }

    /**
     * Get global setting
     *
     * @param string $setting
     *
     * @return string|float|bool|array
     */
    public static function getGlobal($setting)
    {
        $model = self::getModel();
        $model = $model['global'];

        static $globalSettingsCache;

        if (isset($globalSettingsCache[$setting])) {
            $value = $globalSettingsCache[$setting];

            return apply_filters('wcpw_global_setting', $value, $setting);
        }

        $value = isset($model[$setting]['key']) ? get_option($model[$setting]['key'], null) : null;

        if ($value == null && isset($model[$setting]['default'])) {
            $value = $model[$setting]['default'];
        }

        $value = self::handleSettingType($value, $model[$setting]['type']);
        $globalSettingsCache[$setting] = $value;

        return apply_filters('wcpw_global_setting', $value, $setting);
    }

    /**
     * Get one of post setting
     *
     * @param integer $id
     * @param string $setting
     * @param string $modelSource
     *
     * @return string|float|boolean|array
     */
    public static function getPost($id, $setting, $modelSource = 'post')
    {
        $model = self::getModel();
        $model = $model[$modelSource];

        static $postSettingsCache;

        if (isset($postSettingsCache[$id]) && isset($postSettingsCache[$id][$setting])) {
            $value = $postSettingsCache[$id][$setting];

            return apply_filters('wcpw_post_setting', $value, $id, $setting, $modelSource);
        }

        $value = isset($model[$setting]['key']) ? get_post_meta($id, $model[$setting]['key'], true) : null;

        if ($value == null && isset($model[$setting]['default'])) {
            $value = $model[$setting]['default'];
        }

        $value = self::handleSettingType($value, $model[$setting]['type']);

        if (!isset($postSettingsCache[$id])) {
            $postSettingsCache[$id] = [];
        }

        $postSettingsCache[$id][$setting] = $value;

        return apply_filters('wcpw_post_setting', $value, $id, $setting, $modelSource);
    }

    /**
     * Return one post settings array
     *
     * @param integer $id
     * @param array $args
     *
     * @return array
     */
    public static function getPostArray($id, $args = [])
    {
        $defaults = ['public' => false];
        $args = array_merge($defaults, $args);
        $model = self::getModel();
        $model = $model['post'];
        $output = [];

        foreach ($model as $settingModelKey => $settingModel) {
            if ($args['public'] && (!isset($settingModel['public']) || !$settingModel['public'])) {
                continue;
            }

            $output[$settingModelKey] = self::getPost($id, $settingModelKey);
        }

        return apply_filters('wcpw_post_settings', $output, $id, $args);
    }

    /**
     * Get an array of the steps ids which are used in the wizard
     *
     * @param integer $id
     *
     * @return array
     */
    public static function getStepsIds($id)
    {
        static $stepsIdsCache;

        if (!isset($stepsIdsCache[$id])) {
            $stepsIdsCache[$id] = (array) get_post_meta($id, '_steps_ids', 1);
        }

        return apply_filters('wcpw_steps_ids_setting', $stepsIdsCache[$id], $id);
    }

    /**
     * Get steps settings record from DB
     *
     * @param integer $id
     *
     * @return array
     */
    public static function getStepsSettings($id)
    {
        static $stepsSettingsCache;

        if (!isset($stepsSettingsCache[$id])) {
            $stepsSettingsCache[$id] = (array) get_post_meta($id, '_steps_settings', 1);
        }

        return apply_filters('wcpw_step_settings', $stepsSettingsCache[$id], $id);
    }

    /**
     * Get one of wizard step setting
     *
     * @param integer $id
     * @param integer $stepId
     * @param string $setting
     *
     * @return string|float|boolean|array
     */
    public static function getStep($id, $stepId, $setting)
    {
        static $stepSettingsCache;

        if (isset(
            $stepSettingsCache[$id],
            $stepSettingsCache[$id][$stepId],
            $stepSettingsCache[$id][$stepId][$setting]
        )) {
            $value = $stepSettingsCache[$id][$stepId][$setting];

            return apply_filters('wcpw_step_setting', $value, $id, $stepId, $setting);
        }

        $model = self::getModel();
        $model = $model['step'];

        if (!isset($model[$setting])) {
            $stepSettingsCache[$id][$stepId][$setting] = null;

            return apply_filters('wcpw_step_setting', null, $id, $stepId, $setting);
        }

        $meta = self::getStepsSettings($id);

        if ($meta && isset($meta[$stepId]) && isset($meta[$stepId][$model[$setting]['key']])) {
            $value = self::handleSettingType($meta[$stepId][$model[$setting]['key']], $model[$setting]['type']);
            $stepSettingsCache[$id][$stepId][$setting] = $value;

            return apply_filters('wcpw_step_setting', $value, $id, $stepId, $setting);
        }

        if (isset($model[$setting]['default'])) {
            $value = self::handleSettingType($model[$setting]['default'], $model[$setting]['type']);
            $stepSettingsCache[$id][$stepId][$setting] = $value;

            return apply_filters('wcpw_step_setting', $value, $id, $stepId, $setting);
        }

        $value = null;
        $stepSettingsCache[$id][$stepId][$setting] = $value;

        return apply_filters('wcpw_step_setting', $value, $id, $stepId, $setting);
    }

    /**
     * Get product setting
     *
     * @param integer $id
     * @param string $setting
     *
     * @return string|float|bool|array
     */
    public static function getProduct($id, $setting)
    {
        return self::getPost($id, $setting, 'product');
    }

    /**
     * Get final redirect url
     *
     * @param integer $id
     *
     * @return string
     */
    public static function getFinalRedirectUrl($id)
    {
        $url = get_permalink(wc_get_page_id('cart'));

        return apply_filters('wcpw_final_redirect_url', $url, $id);
    }
}
