<?php
namespace WCProductsWizard;

/**
 * WCProductsWizard Settings_Page Class
 *
 * @class Settings_Page
 * @version 1.0.1
 */
// phpcs:disable
class Settings_Page extends \WC_Settings_Page
{
    public $settings = [];

    /**
     * Constructor.
     *
     * @param array $settings
     */
    public function __construct($settings)
    {
        $this->settings = $settings;
        $this->id = 'products_wizard';
        $this->label = L10N::r('Products Wizard', 'woocommerce');

        parent::__construct();
    }

    /**
     * Get sections.
     *
     * @return array
     */
    public function get_sections()
    {
        $sections = array(
            '' => L10N::r('General', 'woocommerce')
        );

        return apply_filters('woocommerce_get_sections_' . $this->id, $sections);
    }

    /**
     * Get settings array.
     *
     * @param string $current_section Current section.
     *
     * @return array
     */
    public function get_settings($current_section = '')
    {
        $settings = [];

        foreach ($this->settings as $key => $setting) {
            if (isset($setting['section']) && $setting['section'] == $current_section) {
                $settings[$key] = $setting;
            }
        }

        return apply_filters('woocommerce_get_settings_' . $this->id, $settings);
    }

    /**
     * Output the settings.
     */
    public function output()
    {
        global $current_section;

        $sections = $this->get_sections();
        $settings = $this->get_settings($current_section);

        woocommerce_admin_fields([
            'section_title' => [
                'name' => isset($sections[$current_section]) ? $sections[$current_section] : 'No title',
                'type' => 'title',
                'desc' => '',
                'id' => 'wcpw_settings_section_title'
            ]
        ]);

        woocommerce_admin_fields($settings);
        woocommerce_admin_fields([
            'section_end' => [
                'type' => 'sectionend',
                'id' => 'wcpw_settings_section_end'
            ]
        ]);
    }

    /**
     * Save settings.
     */
    public function save()
    {
        global $current_section;

        woocommerce_update_options($this->get_settings($current_section));
    }
}
// phpcs:enable
