<?php
namespace WCProductsWizard;

/**
 * WCProductsWizard Template Class
 *
 * @class Template
 * @version 2.3.1
 */
class Template
{
    /**
     * Array of the showed templates
     * @var array
     */
    public static $showed = [];

    /**
     * Include php-template by the name.
     * First looking in the "theme folder/woocommerce-products-wizard (WC_PRODUCTS_WIZARD_THEME_TEMPLATES_DIR)"
     * Second looking in the "plugin folder/views"
     * Making extraction of the arguments as variables
     *
     * @param string $name
     * @param array  $arguments
     * @param array  $settings
     *
     * @return string
     */
    public static function html($name = '', $arguments = [], $settings = [])
    {
        $defaultSettings = [
            'echo' => true,
            'once' => false,
            'compress' => !WC_PRODUCTS_WIZARD_DEBUG
        ];

        $settings = array_merge($defaultSettings, $settings);

        // show template only once
        if ($settings['once'] && in_array($name, self::$showed)) {
            return null;
        }

        // save template as showed
        self::$showed[] = $name;

        if (is_array($arguments)) {
            extract($arguments, EXTR_PREFIX_SAME, 'data');
        }

        $path = get_stylesheet_directory()
            . DIRECTORY_SEPARATOR . WC_PRODUCTS_WIZARD_THEME_TEMPLATES_DIR
            . DIRECTORY_SEPARATOR . $name . '.php';

        if (!file_exists($path)) {
            $path = WC_PRODUCTS_WIZARD_PLUGIN_PATH
                . DIRECTORY_SEPARATOR . 'views'
                . DIRECTORY_SEPARATOR . $name . '.php';
        }

        if (!file_exists($path)) {
            return '';
        }

        ob_start();

        include($path);

        $html = ob_get_clean();

        if ($settings['compress']) {
            $replace = ['>', '<', '\\1', ' ', ''];
            $search = [
                '/\>[^\S ]+/s',     // strip whitespaces after tags, except space
                '/[^\S ]+\</s',     // strip whitespaces before tags, except space
                '/(\s)+/s',         // shorten multiple whitespace sequences
                '/<!--spacer-->/',  // replace spacer tag
                '/<!--(.|\s)*?-->/' // remove HTML comments
            ];

            $html = preg_replace($search, $replace, $html);
        }

        if ($settings['echo']) {
            echo $html;

            return null;
        }

        return $html;
    }
}
