<?php
defined('ABSPATH') || exit;

$id = isset($id) ? $id : null;
$stepId = isset($stepId) ? $stepId : false;

if (!$id) {
    throw new Exception('Empty wizard id');
}

use WCProductsWizard\Form;
use WCProductsWizard\Template;
use WCProductsWizard\Settings;

$defaults = [
    'id' => $id,
    'formId' => null,
    'stepId' => $stepId,
    'mode' => 'free-walk',
    'enableAddToCartButton' => Settings::getPost($id, 'enable_add_to_cart_button'),
    'toResultsButtonEnabled' => Settings::getPost($id, 'enable_to_results_button'),
    'skipButtonEnabled' => Settings::getPost($id, 'enable_skip_button'),
    'backButtonEnabled' => Settings::getPost($id, 'enable_back_button'),
    'resetButtonEnabled' => Settings::getPost($id, 'enable_reset_button'),
    'nextButtonEnabled' => Settings::getPost($id, 'enable_next_button'),
    'canGoBack' => Form::canGoBack($id),
    'canGoForward' => Form::canGoForward($id)
];

$arguments = isset($arguments) ? array_replace($defaults, $arguments) : $defaults;
?>
<div class="woocommerce-products-wizard-controls" data-component="wcpw-controls">
    <?php
    Template::html('controls/widget-toggle', $arguments);

    if ($arguments['mode'] == 'single-step') {
        // is single-step mode
        if ($arguments['resetButtonEnabled']) {
            Template::html('controls/reset', $arguments);
        }

        if ($arguments['enableAddToCartButton']) {
            Template::html('controls/add-to-cart', $arguments);
        }
    } elseif (is_numeric($arguments['stepId']) && $arguments['canGoForward']) {
        // is a numeric step but not last
        if ($arguments['resetButtonEnabled']) {
            Template::html('controls/reset', $arguments);
        }
        
        if ($arguments['backButtonEnabled'] && $arguments['canGoBack']) {
            Template::html('controls/back', $arguments);
        }

        if ($arguments['skipButtonEnabled']) {
            Template::html('controls/skip', $arguments);
        }

        if ($arguments['nextButtonEnabled']) {
            Template::html('controls/next', $arguments);
        }

        Template::html('controls/to-results', $arguments);
    } elseif ($arguments['stepId'] == 'start') {
        // is the start step
        Template::html('controls/start', $arguments);
    } elseif (!$arguments['canGoForward']) {
        // is the last step
        if ($arguments['resetButtonEnabled']) {
            Template::html('controls/reset', $arguments);
        }

        if ($arguments['backButtonEnabled']) {
            Template::html('controls/back', $arguments);
        }

        if ($arguments['enableAddToCartButton']) {
            Template::html('controls/add-to-cart', $arguments);
        }
    }
    ?>
</div>
