<?php
defined('ABSPATH') || exit;

$id = isset($id) ? $id : null;

if (!$id) {
    throw new Exception('Empty wizard id');
}

use WCProductsWizard\Cart;
use WCProductsWizard\Form;
use WCProductsWizard\Product;
use WCProductsWizard\Template;

$activeStepId = Form::getActiveStepId($id);
$stepId = isset($stepId) ? $stepId : $activeStepId;
$step = Form::getStep($id, $stepId);
$pages = [];

if (isset($_REQUEST['wcpwPage'])) {
    if (is_string($_REQUEST['wcpwPage'])) {
        parse_str($_REQUEST['wcpwPage'], $pages);
    } else {
        $pages = $_REQUEST['wcpwPage'];
    }
}

$defaults = [
    'id' => $id,
    'stepId' => $stepId,
    'formId' => null,
    'title' => isset($step['name']) ? $step['name'] : '',
    'description' => isset($step['description']) ? $step['description'] : '',
    'descriptionPosition' => isset($step['descriptionPosition']) ? $step['descriptionPosition'] : 'default',
    'notices' => WCProductsWizard\Instance()->form->getNotices($stepId),
    'page' => isset($pages[$stepId]) ? (int) $pages[$stepId] : 1,
    'mode' => 'free-walk'
];

$arguments = isset($arguments) ? array_replace($defaults, $arguments) : $defaults;
$arguments['stepId'] = $stepId; // force define the current step
$class = ['woocommerce-products-wizard-step', 'woocommerce-products-wizard-form', 'is-step-' . $arguments['stepId']];

if (!empty(Cart::getByStepId($arguments['id'], $arguments['stepId']))) {
    $class[] = 'has-products-in-cart';
}
?>
<article class="<?php echo esc_attr(implode(' ', $class)); ?>"
    data-component="wcpw-form-step" data-id="<?php echo esc_attr($arguments['stepId']); ?>">
    <?php
    if ($arguments['mode'] == 'single-step') {
        Template::html('form/title', $arguments);
    }

    if (!empty($arguments['notices'])) {
        foreach ($arguments['notices'] as $notice) {
            Template::html("messages/{$notice['view']}", array_replace($arguments, $notice));
        }
    }

    if ($arguments['description'] && in_array($arguments['descriptionPosition'], ['top', 'default'])) {
        Template::html('form/description', $arguments);
    }

    if (is_numeric($stepId)) {
        ?>
        <input type="hidden"
            form="<?php echo esc_attr($arguments['formId']); ?>"
            name="productsToAddChecked[<?php echo esc_attr($arguments['stepId']); ?>][]"
            value="">
        <?php
        Product::request((int) $id, $stepId, $arguments);
    }

    if ($arguments['description'] && $arguments['descriptionPosition'] == 'bottom') {
        Template::html('form/description', $arguments);
    }
    ?>
</article>
