<?php
defined('ABSPATH') || exit;

use WCProductsWizard\Utils;

$thumbnailLink = wp_get_attachment_image_src(get_post_thumbnail_id(), 'large');
$defaults = [
    'class' => 'woocommerce-products-wizard-form-item',
    'enableThumbnailLink' => true,
    'thumbnailSize' => 'shop_catalog',
    'thumbnailLink' => isset($thumbnailLink[0]) ? $thumbnailLink[0] : '',
    'thumbnailAttributes' => ['data-component' => 'wcpw-product-thumbnail-image'],
    'product' => null
];

$arguments = isset($arguments) ? array_replace_recursive($defaults, $arguments) : $defaults;

$dimensions = wc_get_image_size($arguments['thumbnailSize']);
$placeholder = '<img src="' . wc_placeholder_img_src()
    . '" alt="' . esc_html__('Placeholder', 'woocommerce')
    . '" width="' . esc_attr($dimensions['width'])
    . '" height="' . esc_attr($dimensions['height']) . '" '
    . Utils::attributesArrayToString($arguments['thumbnailAttributes'])
    . '/>';
?>
<figure class="<?php echo esc_attr($arguments['class']); ?>-thumbnail thumbnail img-thumbnail"
    data-component="wcpw-product-thumbnail"><?php
    if ($arguments['thumbnailLink'] && $arguments['enableThumbnailLink']) {
        echo '<a href="' . esc_attr($arguments['thumbnailLink']) . '"
            class="' . esc_attr($arguments['class']) . '-thumbnail-link"
            title="' . esc_attr(get_the_title(get_post_thumbnail_id())) . '"
            data-component="wcpw-product-thumbnail-link"
            data-rel="prettyPhoto[product-gallery-' . esc_attr($arguments['product']->get_id()) . ']"
            rel="lightbox[' . esc_attr($arguments['product']->get_id()) . ']">';
    }

    echo has_post_thumbnail()
        ? get_the_post_thumbnail(
            $arguments['product']->get_id(),
            $arguments['thumbnailSize'],
            $arguments['thumbnailAttributes']
        )
        : $placeholder;

    if ($arguments['thumbnailLink'] && $arguments['enableThumbnailLink']) {
        echo '</a>';
    }
    ?></figure>
