<?php
defined('ABSPATH') || exit;

$id = isset($id) ? $id : null;

if (!$id) {
    throw new Exception('Empty wizard id');
}

use WCProductsWizard\Cart;
use WCProductsWizard\Template;
use WCProductsWizard\Settings;

$id = isset($id) ? $id : null;
$singleStepMode = isset($singleStepMode) ? $singleStepMode : false;
$notices = WCProductsWizard\Instance()->form->getNotices($singleStepMode ? 'result' : null);

$defaults = [
    'id' => $id,
    'cart' => Cart::get($id),
    'notices' => $notices,
    'enableRemoveButton' => Settings::getPost($id, 'enable_remove_button'),
    'removeButtonText' => Settings::getPost($id, 'remove_button_text'),
    'removeString' => Settings::getPost($id, 'results_remove_string'),
    'priceString' => Settings::getPost($id, 'results_price_string'),
    'thumbnailString' => Settings::getPost($id, 'results_thumbnail_string'),
    'dataString'=> Settings::getPost($id, 'results_data_string'),
    'quantityString'=> Settings::getPost($id, 'results_quantity_string'),
    'subtotalString'=> Settings::getPost($id, 'subtotal_string'),
    'totalString'=> Settings::getPost($id, 'total_string')
];

$arguments = isset($arguments) ? array_replace($defaults, $arguments) : $defaults;

$showProductsHeader = true;
$captionColspan = 4 + (int) $arguments['enableRemoveButton'];

if (!empty($arguments['notices'])) {
    foreach ($arguments['notices'] as $notice) {
        Template::html("messages/{$notice['view']}", array_replace($arguments, $notice));
    }
}

if (empty($arguments['cart'])) {
    return Template::html('messages/cart-is-empty', $arguments);
}
?>
<div class="woocommerce-products-wizard-results-table-wrapper">
    <table class="woocommerce-products-wizard-results-table table table-hover"
        data-component="wcpw-results-table">
        <tbody class="woocommerce-products-wizard-results-table-body">
            <tr class="woocommerce-products-wizard-results-table-body-row is-products">
                <?php if ($arguments['enableRemoveButton']) { ?>
                    <th class="woocommerce-products-wizard-results-table-header-cell is-remove">
                        <span class="sr-only"><?php echo wp_kses_post($arguments['removeString']); ?></span>
                    </th>
                <?php } ?>
                <th class="woocommerce-products-wizard-results-table-header-cell is-thumbnail">
                    <span class="sr-only"><?php echo wp_kses_post($arguments['thumbnailString']); ?></span>
                </th>
                <th class="woocommerce-products-wizard-results-table-header-cell is-data"><?php
                    echo wp_kses_post($arguments['dataString']);
                    ?></th>
                <th class="woocommerce-products-wizard-results-table-header-cell is-price"><?php
                    echo wp_kses_post($arguments['priceString']);
                    ?></th>
                <th class="woocommerce-products-wizard-results-table-header-cell is-quantity"><?php
                    echo wp_kses_post($arguments['quantityString']);
                    ?></th>
                <th class="woocommerce-products-wizard-results-table-header-cell is-subtotal"><?php
                    echo wp_kses_post($arguments['subtotalString']);
                    ?></th>
            </tr>
            <?php
            foreach ($arguments['cart'] as $cartItemKey => $cartItem) {
                if ((isset($cartItem['data']) && (!$cartItem['data'] || !$cartItem['data']->exists()))
                    || (isset($cartItem['quantity']) && $cartItem['quantity'] <= 0)
                    || (isset($cartItem['value']) && empty($cartItem['value']))
                    || !isset($cartItem['step_id'])
                ) {
                    continue;
                }
                ?>
                <tr class="<?php
                    echo esc_attr("woocommerce-products-wizard-results-table-body-row is-step-{$cartItem['step_id']}");
                    ?>">
                    <?php
                    if (isset($cartItem['product_id'], $cartItem['data']) && $cartItem['data']) {
                        $product = $cartItem['data'];
                        ?>
                        <?php if ($arguments['enableRemoveButton']) { ?>
                            <td class="woocommerce-products-wizard-results-table-body-cell is-remove"
                                data-th="<?php echo esc_attr($arguments['removeString']); ?>">
                                <button class="close woocommerce-products-wizard-results-item-remove"
                                    aria-label="<?php echo esc_attr($arguments['removeString']); ?>"
                                    form="<?php echo esc_attr($arguments['formId']); ?>"
                                    name="remove-cart-product"
                                    value="<?php echo esc_attr($cartItemKey); ?>"
                                    title="<?php echo esc_attr($arguments['removeButtonText']); ?>"
                                    data-component="wcpw-remove-cart-product">
                                    <span aria-hidden="true">&times;</span>
                                    <span class="woocommerce-products-wizard-results-item-remove-text"><?php
                                        echo wp_kses_post($arguments['removeButtonText']);
                                        ?></span>
                                </button>
                            </td>
                        <?php } ?>
                        <td class="woocommerce-products-wizard-results-table-body-cell is-thumbnail"
                            data-th="<?php echo esc_attr($arguments['thumbnailString']); ?>">
                            <figure class="woocommerce-products-wizard-results-item-thumbnail"><?php
                                $href = wp_get_attachment_image_src($product->get_image_id(), 'full');
                                $thumbnail = apply_filters(
                                    'wcpw_result_item_thumbnail',
                                    $product->get_image('shop_thumbnail', ['class' => 'img-thumbnail']),
                                    $cartItem,
                                    $cartItemKey
                                );

                                echo isset($href[0])
                                    ? "<a href=\"{$href[0]}\" data-rel=\"prettyPhoto\" rel=\"lightbox\">{$thumbnail}</a>"
                                    : $thumbnail;
                                ?></figure>
                        </td>
                        <td class="woocommerce-products-wizard-results-table-body-cell is-data"
                            data-th="<?php echo esc_attr($arguments['dataString']); ?>">
                            <div class="woocommerce-products-wizard-results-item-title"><?php
                                if (method_exists($product, 'get_name')) {
                                    echo $product->get_name();
                                }
                                ?></div>
                            <div class="woocommerce-products-wizard-results-item-data"><?php
                                // phpcs:disable
                                // Localization variations
                                $cartItemLocalized = $cartItem;

                                if (isset($cartItem['variation']) && is_array($cartItem['variation'])) {
                                    foreach ($cartItemLocalized['variation'] as &$variationsItem) {
                                        $variationsItem = urldecode($variationsItem);
                                    }
                                }

                                if (function_exists('wc_get_formatted_cart_item_data')) {
                                    echo wc_get_formatted_cart_item_data($cartItemLocalized);
                                } else {
                                    echo \WC()->cart->get_item_data($cartItemLocalized);
                                }

                                // Backorder notification
                                if ($product->backorders_require_notification()
                                    && $product->is_on_backorder($cartItem['quantity'])
                                ) {
                                    echo '<p class="backorder_notification">'
                                        . esc_html__(
                                            'Available on backorder',
                                            'woocommerce'
                                        )
                                        . '</p>';
                                }
                                // phpcs:enable
                                ?></div>
                        </td>
                        <td class="woocommerce-products-wizard-results-table-body-cell is-price"
                            data-th="<?php echo esc_attr($arguments['priceString']); ?>">
                            <span class="woocommerce-products-wizard-results-item-price<?php
                            echo $product->get_price() == 0 ? ' is-zero-price ' : '';
                            ?>"><?php echo \WC()->cart->get_product_price($product); ?></span>
                        </td>
                        <td class="woocommerce-products-wizard-results-table-body-cell is-quantity"
                            data-th="<?php echo esc_attr($arguments['quantityString']); ?>">
                        <span class="woocommerce-products-wizard-results-item-quantity"><?php
                            echo $cartItem['quantity'];
                            ?></span>
                        </td>
                        <td class="woocommerce-products-wizard-results-table-body-cell is-subtotal"
                            data-th="<?php echo esc_attr($arguments['subtotalString']); ?>">
                            <span class="woocommerce-products-wizard-results-item-subtotal<?php
                            echo $product->get_price() == 0 ? ' is-zero-price ' : '';
                            ?>"><?php
                                echo WC()->cart->get_product_subtotal($product, $cartItem['quantity']);
                                ?></span>
                        </td>
                        <?php
                    }
                    ?>
                </tr>
                <?php
            }
            ?>
        </tbody>
        <tfoot class="woocommerce-products-wizard-results-table-footer">
            <tr class="woocommerce-products-wizard-results-table-footer-row is-total">
                <th class="woocommerce-products-wizard-results-table-footer-cell is-caption"
                        colspan="<?php echo $captionColspan; ?>"><?php
                    echo wp_kses_post($arguments['totalString']);
                    ?></th>
                <td class="woocommerce-products-wizard-results-table-footer-cell is-value"
                    data-th="<?php echo esc_attr($arguments['totalString']); ?>"><?php
                    echo wc_price(Cart::getTotal($arguments['id']));
                    ?></td>
            </tr>
        </tfoot>
    </table>
</div>
