<?php
defined('ABSPATH') || exit;

use WCProductsWizard\Cart;
use WCProductsWizard\Template;
use WCProductsWizard\Settings;

$id = isset($id) ? $id : null;

if (!$id) {
    throw new Exception('Empty wizard id');
}

$defaults = [
    'id' => $id,
    'cart' => Cart::get($id),
    'formId' => null,
    'enableRemoveButton' => Settings::getPost($id, 'enable_remove_button'),
    'removeButtonText' => Settings::getPost($id, 'remove_button_text'),
    'stickyOffsetTop' => Settings::getPost($id, 'sticky_sidebar_offset_top'),
    'subtotalString'=> Settings::getPost($id, 'subtotal_string'),
    'totalString'=> Settings::getPost($id, 'total_string'),
];

$arguments = isset($arguments) ? array_replace($defaults, $arguments) : $defaults;
?>
<section class="woocommerce-products-wizard-widget"
    id="woocommerce-products-wizard-widget-<?php echo esc_attr($arguments['id']); ?>"
    aria-expanded="false"
    data-component="wcpw-widget wcpw-sticky"
    data-sticky-top-offset="<?php echo esc_attr($arguments['stickyOffsetTop']); ?>"
    data-sticky-parent="[data-component=wcpw-main-row]">
    <?php
    if (empty($arguments['cart'])) {
        Template::html('messages/cart-is-empty', $arguments);
    } else {
        ?>
        <table class="woocommerce-products-wizard-widget-table table table-bordered table-hover">
            <tbody class="woocommerce-products-wizard-widget-table-body">
                <?php
                foreach ($arguments['cart'] as $cartItemKey => $cartItem) {
                    if ((isset($cartItem['data']) && (!$cartItem['data'] || !$cartItem['data']->exists()))
                        || (isset($cartItem['quantity']) && $cartItem['quantity'] <= 0)
                        || (isset($cartItem['value']) && empty($cartItem['value']))
                        || !isset($cartItem['step_id'])
                    ) {
                        continue;
                    }
                    ?>
                    <tr class="<?php
                        echo esc_attr("woocommerce-products-wizard-widget-table-body-row is-step-{$cartItem['step_id']}");
                        ?>">
                        <?php
                        if (isset($cartItem['product_id'], $cartItem['data']) && !is_null($cartItem['data'])) {
                            $product = $cartItem['data'];
                            ?>
                            <td class="woocommerce-products-wizard-widget-table-body-cell is-thumbnail">
                                <figure class="woocommerce-products-wizard-widget-item-thumbnail">
                                    <?php
                                    $href = wp_get_attachment_image_src($product->get_image_id(), 'large');
                                    $thumbnail = apply_filters(
                                        'wcpw_widget_item_thumbnail',
                                        $product->get_image('shop_thumbnail', ['class' => 'img-thumbnail']),
                                        $cartItem,
                                        $cartItemKey
                                    );

                                    echo isset($href[0])
                                        ? "<a href=\"{$href[0]}\" data-rel=\"prettyPhoto\" rel=\"lightbox\">{$thumbnail}</a>"
                                        : $thumbnail;
                                    ?>
                                </figure>
                            </td>
                            <td class="woocommerce-products-wizard-widget-table-body-cell is-data">
                                <div class="woocommerce-products-wizard-widget-item-title"><?php
                                    if (method_exists($product, 'get_name')) {
                                        echo $product->get_name();
                                    }
                                    ?></div>
                                <div class="woocommerce-products-wizard-widget-item-data">
                                    <?php
                                    // Localization variations
                                    $cartItemLocalized = $cartItem;

                                    if (isset($cartItem['variation']) && is_array($cartItem['variation'])) {
                                        foreach ($cartItemLocalized['variation'] as &$variationsItem) {
                                            $variationsItem = urldecode($variationsItem);
                                        }
                                    }

                                    if (function_exists('wc_get_formatted_cart_item_data')) {
                                        echo wc_get_formatted_cart_item_data($cartItemLocalized);
                                    } else {
                                        echo \WC()->cart->get_item_data($cartItemLocalized);
                                    }
                                    ?>
                                </div>
                                <span class="woocommerce-products-wizard-widget-item-price<?php
                                    echo $product->get_price() == 0 ? ' is-zero-price ' : '';
                                    ?>">
                                    <?php
                                    echo \WC()->cart->get_product_price($product);

                                    if (!isset($cartItem['sold_individually']) || !$cartItem['sold_individually']) {
                                        ?>
                                        <bdi class="woocommerce-products-wizard-widget-item-price-times">x</bdi>
                                        <span class="woocommerce-products-wizard-widget-item-price-quantity"><?php
                                            echo $cartItem['quantity'];
                                            ?></span>
                                        <?php
                                    }
                                    ?>
                                </span>
                                <?php if ($arguments['enableRemoveButton']) { ?>
                                    <button class="btn-light btn-sm show-icon icon-left hide-text btn woocommerce-products-wizard-widget-item-control woocommerce-products-wizard-control is-remove-from-cart"
                                        form="<?php echo esc_attr($arguments['formId']); ?>"
                                        name="remove-cart-product"
                                        value="<?php echo esc_attr($cartItemKey); ?>"
                                        title="<?php echo esc_attr($arguments['removeButtonText']); ?>"
                                        data-component="wcpw-remove-cart-product">
                                        <!--spacer-->
                                        <span class="woocommerce-products-wizard-control-inner"><?php
                                            echo wp_kses_post($arguments['removeButtonText']);
                                            ?></span>
                                        <!--spacer-->
                                    </button>
                                <?php } ?>
                            </td>
                            <?php
                        } elseif (isset($cartItem['value'], $cartItem['key']) && !empty($cartItem['value'])
                            && !empty($cartItem['key'])
                        ) {
                            ?>
                            <th class="woocommerce-products-wizard-widget-table-body-cell is-name">
                                <span class="woocommerce-products-wizard-widget-item-name"><?php
                                    echo wp_kses_post($cartItem['key']);
                                    ?></span>
                            </th>
                            <td class="woocommerce-products-wizard-widget-table-body-cell is-value">
                                <span class="woocommerce-products-wizard-widget-item-value"><?php
                                    echo wp_kses_post($cartItem['value']);
                                    ?></span>
                            </td>
                            <?php
                        }
                        ?>
                    </tr>
                    <?php
                }
                ?>
            </tbody>
            <tfoot class="woocommerce-products-wizard-widget-table-footer">
                <tr class="woocommerce-products-wizard-widget-table-footer-row is-total">
                    <th class="woocommerce-products-wizard-widget-table-footer-cell is-caption"><?php
                        echo wp_kses_post($arguments['totalString']);
                        ?></th>
                    <td class="woocommerce-products-wizard-widget-table-footer-cell is-value"><?php
                        echo wc_price(Cart::getTotal($arguments['id']));
                        ?></td>
                </tr>
            </tfoot>
        </table>
        <?php
    }
    ?>
</section>
